

local Spyglass = Spyglass
local L = AceLibrary("AceLocale-2.2"):new("Spyglass")
local IBL = AceLibrary('ItemBonusLib-1.0')
local Crayon = AceLibrary('Crayon-2.0')


local bonusCategories = {
    {
        name = L['Attributes'],
        effects = {
            { effect = "STR" },
            { effect = "AGI" },
            { effect = "STA" },
            { effect = "INT" },
            { effect = "SPI" },
        }
    },

    {
        name = L['Resistance'],
        effects = {
            { effect = "ARCANERES" },
            { effect = "FIRERES"   },
            { effect = "NATURERES" },
            { effect = "FROSTRES"  },
            { effect = "SHADOWRES" },
        },
    },

    {
        name = L['Skills'],
        effects = {
            { effect = "FISHING"   },
            { effect = "MINING"    },
            { effect = "HERBALISM" },
            { effect = "SKINNING"  },
        },
    },

    {
        name = L['Combat'],
        effects = {
            { effect = "CR_WEAPON" },
            { effect = "CR_WEAPON_AXE" },
            { effect = "CR_WEAPON_DAGGER" },
            { effect = "CR_WEAPON_MACE" },
            { effect = "CR_WEAPON_SWORD" },
            { effect = "CR_WEAPON_SWORD_2H" },
            
            { effect = "WEPDMG" },
            { effect = "CR_HASTE" },
            { effect = "ATTACKPOWER" },
            { effect = "ATTACKPOWERUNDEAD", },
            { effect = "ATTACKPOWERBEAST" },
            { effect = "ATTACKPOWERDEMON" },
            { effect = "ATTACKPOWERFERAL" },
            { effect = "RANGEDATTACKPOWER" },
            
            { effect = "TOHIT", format = "+%d%%" },
            { effect = "CR_HIT" },
            { effect = "CRIT", format = "+%d%%" },
            { effect = "CR_CRIT" },
            { effect = "RANGEDCRIT", format = "+%d%%" },
            
            { effect = "IGNOREARMOR" },
        },
    },
    
    {
        name = L['Defenses'],
        effects = {
            { effect = "ARMOR" },
            { effect = "BASE_ARMOR" },
            { effect = "ARMOR_BONUS" },
            { effect = "DEFENSE" },
            { effect = "CR_DEFENSE" },
            { effect = "DODGE", format = "+%d%%" },
            { effect = "CR_DODGE" },
            { effect = "PARRY", format = "+%d%%" },
            { effect = "CR_PARRY" },
            { effect = "BLOCK", format = "+%d%%" },
            { effect = "CR_BLOCK" },
            { effect = "BLOCKAMT" },
            { effect = "CR_RESILIENCE" },
        },
    },

    {
        name = L['Spells'],
        effects = {
            { effect = "DMG" },
            { effect = "DMGUNDEAD" },
            { effect = "ARCANEDMG" },
            { effect = "FIREDMG" },
            { effect = "FROSTDMG" },
            { effect = "HOLYDMG" },
            { effect = "NATUREDMG" },
            { effect = "SHADOWDMG" },
            { effect = "HEAL" },
            { effect = "SPELLTOHIT", format = "%d%%" },
            { effect = "CR_SPELLHIT" },
            { effect = "SPELLCRIT", format = "%d%%" },
            { effect = "CR_SPELLCRIT" },
            { effect = "HOLYCRIT", format = "%d%%" },
            { effect = "SPELLPEN" },
            { effect = "CR_SPELLHASTE" },
        },
    },

    {
        name = L['Health and Mana'],
        effects = {
            { effect = "HEALTH" },
            { effect = "HEALTHREG", format = L["%d hp5"] },
            { effect = "MANA" },
            { effect = "MANAREG", format = L["%d mp5"] },
        },
    },
}

function Spyglass:BuildBonusText(unitBonuses)
    if not unitBonuses then
        self:Print(L['bonuses not passed?'])
        return ''
    end
    
    local bonusTableL = {}
    local bonusTableR = {}
    local currentCategory = ''
    local bonusName, bonusValue
    local eformat
    
    for i, category in ipairs(bonusCategories) do
        for i, effect in ipairs(category.effects) do
            bonusValue = unitBonuses[effect.effect]
            
            if bonusValue ~= nil then
                bonusName = IBL:GetBonusFriendlyName(effect.effect)
                
                eformat = effect.format or '+%d'
                local oldformat = effect.format or 'nil'
                bonusValue = eformat:format(bonusValue)
                
                if category.name ~= currentCategory then
                    currentCategory = category.name
                    
                    if #bonusTableL > 0 then
                        table.insert(bonusTableL, '')
                        table.insert(bonusTableR, '')
                    end
                    
                    table.insert(bonusTableL, Crayon:Yellow(currentCategory))
                    table.insert(bonusTableR, '')
                end
                
                table.insert(bonusTableL, Crayon:Gold(bonusName..':'))
                table.insert(bonusTableR, Crayon:Green(bonusValue))
            end
        end
    end
    
    return table.concat(bonusTableL, '\n'), table.concat(bonusTableR, '\n')
end
